@@ -46,7 +46,7 @@ class BaseModelAdmin(object):
 
         If kwargs are given, they're passed to the form Field's constructor.
         """
-        
+
         # If the field specifies choices, we don't need to look for special
         # admin widgets - we just need to use a select widget of some kind.
         if db_field.choices:
@@ -79,17 +79,17 @@ class BaseModelAdmin(object):
         if isinstance(db_field, models.TimeField):
             kwargs['widget'] = widgets.AdminTimeWidget
             return db_field.formfield(**kwargs)
-        
+
         # For TextFields, add a custom CSS class.
         if isinstance(db_field, models.TextField):
             kwargs['widget'] = widgets.AdminTextareaWidget
             return db_field.formfield(**kwargs)
-        
+
         # For URLFields, add a custom CSS class.
         if isinstance(db_field, models.URLField):
             kwargs['widget'] = widgets.AdminURLFieldWidget
             return db_field.formfield(**kwargs)
-        
+
         # For IntegerFields, add a custom CSS class.
         if isinstance(db_field, models.IntegerField):
             kwargs['widget'] = widgets.AdminIntegerFieldWidget
@@ -104,7 +104,7 @@ class BaseModelAdmin(object):
         if isinstance(db_field, models.CharField):
             kwargs['widget'] = widgets.AdminTextInputWidget
             return db_field.formfield(**kwargs)
-    
+
         # For FileFields and ImageFields add a link to the current file.
         if isinstance(db_field, models.ImageField) or isinstance(db_field, models.FileField):
             kwargs['widget'] = widgets.AdminFileWidget
@@ -266,11 +266,13 @@ class ModelAdmin(BaseModelAdmin):
         if self.exclude is None:
             exclude = []
         else:
-            exclude = self.exclude
+            exclude = list(self.exclude)
+        # if exclude is an empty list we pass None to be consistant with the
+        # default on modelform_factory
         defaults = {
             "form": self.form,
             "fields": fields,
-            "exclude": exclude + kwargs.get("exclude", []),
+            "exclude": (exclude + kwargs.get("exclude", [])) or None,
             "formfield_callback": self.formfield_for_dbfield,
         }
         defaults.update(kwargs)
@@ -279,56 +281,56 @@ class ModelAdmin(BaseModelAdmin):
     def get_formsets(self, request, obj=None):
         for inline in self.inline_instances:
             yield inline.get_formset(request, obj)
-            
+
     def log_addition(self, request, object):
         """
-        Log that an object has been successfully added. 
-        
+        Log that an object has been successfully added.
+
         The default implementation creates an admin LogEntry object.
         """
         from django.contrib.admin.models import LogEntry, ADDITION
         LogEntry.objects.log_action(
-            user_id         = request.user.pk, 
+            user_id         = request.user.pk,
             content_type_id = ContentType.objects.get_for_model(object).pk,
             object_id       = object.pk,
-            object_repr     = force_unicode(object), 
+            object_repr     = force_unicode(object),
             action_flag     = ADDITION
         )
-        
+
     def log_change(self, request, object, message):
         """
-        Log that an object has been successfully changed. 
-        
+        Log that an object has been successfully changed.
+
         The default implementation creates an admin LogEntry object.
         """
         from django.contrib.admin.models import LogEntry, CHANGE
         LogEntry.objects.log_action(
-            user_id         = request.user.pk, 
-            content_type_id = ContentType.objects.get_for_model(object).pk, 
-            object_id       = object.pk, 
-            object_repr     = force_unicode(object), 
-            action_flag     = CHANGE, 
+            user_id         = request.user.pk,
+            content_type_id = ContentType.objects.get_for_model(object).pk,
+            object_id       = object.pk,
+            object_repr     = force_unicode(object),
+            action_flag     = CHANGE,
             change_message  = message
         )
-        
+
     def log_deletion(self, request, object, object_repr):
         """
         Log that an object has been successfully deleted. Note that since the
         object is deleted, it might no longer be safe to call *any* methods
         on the object, hence this method getting object_repr.
-        
+
         The default implementation creates an admin LogEntry object.
         """
         from django.contrib.admin.models import LogEntry, DELETION
         LogEntry.objects.log_action(
-            user_id         = request.user.id, 
-            content_type_id = ContentType.objects.get_for_model(self.model).pk, 
-            object_id       = object.pk, 
+            user_id         = request.user.id,
+            content_type_id = ContentType.objects.get_for_model(self.model).pk,
+            object_id       = object.pk,
             object_repr     = object_repr,
             action_flag     = DELETION
         )
-        
-    
+
+
     def construct_change_message(self, request, form, formsets):
         """
         Construct a change message from a changed object.
@@ -342,22 +344,22 @@ class ModelAdmin(BaseModelAdmin):
                 for added_object in formset.new_objects:
                     change_message.append(_('Added %(name)s "%(object)s".')
                                           % {'name': added_object._meta.verbose_name,
-                                             'object': added_object})
+                                             'object': force_unicode(added_object)})
                 for changed_object, changed_fields in formset.changed_objects:
                     change_message.append(_('Changed %(list)s for %(name)s "%(object)s".')
                                           % {'list': get_text_list(changed_fields, _('and')),
                                              'name': changed_object._meta.verbose_name,
-                                             'object': changed_object})
+                                             'object': force_unicode(changed_object)})
                 for deleted_object in formset.deleted_objects:
                     change_message.append(_('Deleted %(name)s "%(object)s".')
                                           % {'name': deleted_object._meta.verbose_name,
-                                             'object': deleted_object})
+                                             'object': force_unicode(deleted_object)})
         change_message = ' '.join(change_message)
         return change_message or _('No fields changed.')
-    
+
     def message_user(self, request, message):
         """
-        Send a message to the user. The default implementation 
+        Send a message to the user. The default implementation
         posts a message using the auth Message object.
         """
         request.user.message_set.create(message=message)
@@ -368,7 +370,7 @@ class ModelAdmin(BaseModelAdmin):
         the object is being changed, and False if it's being added.
         """
         return form.save(commit=False)
-    
+
     def save_model(self, request, obj, form, change):
         """
         Given a model instance save it to the database.
@@ -406,14 +408,14 @@ class ModelAdmin(BaseModelAdmin):
             "admin/%s/change_form.html" % app_label,
             "admin/change_form.html"
         ], context, context_instance=template.RequestContext(request))
-    
+
     def response_add(self, request, obj, post_url_continue='../%s/'):
         """
         Determines the HttpResponse for the add_view stage.
         """
         opts = obj._meta
         pk_value = obj._get_pk_val()
-        
+
         msg = _('The %(name)s "%(obj)s" was added successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
         # Here, we distinguish between different save types by checking for
         # the presence of keys in request.POST.
@@ -422,7 +424,7 @@ class ModelAdmin(BaseModelAdmin):
             if request.POST.has_key("_popup"):
                 post_url_continue += "?_popup=1"
             return HttpResponseRedirect(post_url_continue % pk_value)
-        
+
         if request.POST.has_key("_popup"):
             return HttpResponse('<script type="text/javascript">opener.dismissAddAnotherPopup(window, "%s", "%s");</script>' % \
                 # escape() calls force_unicode.
@@ -441,14 +443,14 @@ class ModelAdmin(BaseModelAdmin):
             else:
                 post_url = '../../../'
             return HttpResponseRedirect(post_url)
-    
+
     def response_change(self, request, obj):
         """
         Determines the HttpResponse for the change_view stage.
         """
         opts = obj._meta
         pk_value = obj._get_pk_val()
-        
+
         msg = _('The %(name)s "%(obj)s" was changed successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
         if request.POST.has_key("_continue"):
             self.message_user(request, msg + ' ' + _("You may edit it again below."))
@@ -471,18 +473,10 @@ class ModelAdmin(BaseModelAdmin):
         "The 'add' admin view for this model."
         model = self.model
         opts = model._meta
-        app_label = opts.app_label
 
         if not self.has_add_permission(request):
             raise PermissionDenied
 
-        if self.has_change_permission(request, None):
-            # redirect to list view
-            post_url = '../'
-        else:
-            # Object list will give 'Permission Denied', so go back to admin home
-            post_url = '../../../'
-
         ModelForm = self.get_form(request)
         formsets = []
         if request.method == 'POST':
@@ -493,17 +487,23 @@ class ModelAdmin(BaseModelAdmin):
             else:
                 form_validated = False
                 new_object = self.model()
+            prefixes = {}
             for FormSet in self.get_formsets(request):
+                prefix = FormSet.get_default_prefix()
+                prefixes[prefix] = prefixes.get(prefix, 0) + 1
+                if prefixes[prefix] != 1:
+                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                 formset = FormSet(data=request.POST, files=request.FILES,
                                   instance=new_object,
-                                  save_as_new=request.POST.has_key("_saveasnew"))
+                                  save_as_new=request.POST.has_key("_saveasnew"),
+                                  prefix=prefix)
                 formsets.append(formset)
             if all_valid(formsets) and form_validated:
                 self.save_model(request, new_object, form, change=False)
                 form.save_m2m()
                 for formset in formsets:
                     self.save_formset(request, form, formset, change=False)
-                
+
                 self.log_addition(request, new_object)
                 return self.response_add(request, new_object)
         else:
@@ -518,8 +518,13 @@ class ModelAdmin(BaseModelAdmin):
                 if isinstance(f, models.ManyToManyField):
                     initial[k] = initial[k].split(",")
             form = ModelForm(initial=initial)
+            prefixes = {}
             for FormSet in self.get_formsets(request):
-                formset = FormSet(instance=self.model())
+                prefix = FormSet.get_default_prefix()
+                prefixes[prefix] = prefixes.get(prefix, 0) + 1
+                if prefixes[prefix] != 1:
+                    prefix = "%s-%s" % (prefix, prefixes[prefix])
+                formset = FormSet(instance=self.model(), prefix=prefix)
                 formsets.append(formset)
 
         adminForm = helpers.AdminForm(form, list(self.get_fieldsets(request)), self.prepopulated_fields)
@@ -541,20 +546,19 @@ class ModelAdmin(BaseModelAdmin):
             'inline_admin_formsets': inline_admin_formsets,
             'errors': helpers.AdminErrorList(form, formsets),
             'root_path': self.admin_site.root_path,
-            'app_label': app_label,
+            'app_label': opts.app_label,
         }
         context.update(extra_context or {})
-        return self.render_change_form(request, context, add=True)
+        return self.render_change_form(request, context, form_url=form_url, add=True)
     add_view = transaction.commit_on_success(add_view)
 
     def change_view(self, request, object_id, extra_context=None):
         "The 'change' admin view for this model."
         model = self.model
         opts = model._meta
-        app_label = opts.app_label
 
         try:
-            obj = model._default_manager.get(pk=object_id)
+            obj = self.queryset(request).get(pk=object_id)
         except model.DoesNotExist:
             # Don't raise Http404 just yet, because we haven't checked
             # permissions yet. We don't want an unauthenticated user to be able
@@ -565,10 +569,10 @@ class ModelAdmin(BaseModelAdmin):
             raise PermissionDenied
 
         if obj is None:
-            raise Http404('%s object with primary key %r does not exist.' % (force_unicode(opts.verbose_name), escape(object_id)))
+            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
 
-        if request.POST and request.POST.has_key("_saveasnew"):
-            return self.add_view(request, form_url='../../add/')
+        if request.method == 'POST' and request.POST.has_key("_saveasnew"):
+            return self.add_view(request, form_url='../add/')
 
         ModelForm = self.get_form(request, obj)
         formsets = []
@@ -580,9 +584,14 @@ class ModelAdmin(BaseModelAdmin):
             else:
                 form_validated = False
                 new_object = obj
+            prefixes = {}
             for FormSet in self.get_formsets(request, new_object):
+                prefix = FormSet.get_default_prefix()
+                prefixes[prefix] = prefixes.get(prefix, 0) + 1
+                if prefixes[prefix] != 1:
+                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                 formset = FormSet(request.POST, request.FILES,
-                                  instance=new_object)
+                                  instance=new_object, prefix=prefix)
                 formsets.append(formset)
 
             if all_valid(formsets) and form_validated:
@@ -590,14 +599,20 @@ class ModelAdmin(BaseModelAdmin):
                 form.save_m2m()
                 for formset in formsets:
                     self.save_formset(request, form, formset, change=True)
-                
+
                 change_message = self.construct_change_message(request, form, formsets)
                 self.log_change(request, new_object, change_message)
                 return self.response_change(request, new_object)
+
         else:
             form = ModelForm(instance=obj)
+            prefixes = {}
             for FormSet in self.get_formsets(request, obj):
-                formset = FormSet(instance=obj)
+                prefix = FormSet.get_default_prefix()
+                prefixes[prefix] = prefixes.get(prefix, 0) + 1
+                if prefixes[prefix] != 1:
+                    prefix = "%s-%s" % (prefix, prefixes[prefix])
+                formset = FormSet(instance=obj, prefix=prefix)
                 formsets.append(formset)
 
         adminForm = helpers.AdminForm(form, self.get_fieldsets(request, obj), self.prepopulated_fields)
@@ -620,7 +635,7 @@ class ModelAdmin(BaseModelAdmin):
             'inline_admin_formsets': inline_admin_formsets,
             'errors': helpers.AdminErrorList(form, formsets),
             'root_path': self.admin_site.root_path,
-            'app_label': app_label,
+            'app_label': opts.app_label,
         }
         context.update(extra_context or {})
         return self.render_change_form(request, context, change=True, obj=obj)
@@ -667,7 +682,7 @@ class ModelAdmin(BaseModelAdmin):
         app_label = opts.app_label
 
         try:
-            obj = self.model._default_manager.get(pk=object_id)
+            obj = self.queryset(request).get(pk=object_id)
         except self.model.DoesNotExist:
             # Don't raise Http404 just yet, because we haven't checked
             # permissions yet. We don't want an unauthenticated user to be able
@@ -678,7 +693,7 @@ class ModelAdmin(BaseModelAdmin):
             raise PermissionDenied
 
         if obj is None:
-            raise Http404('%s object with primary key %r does not exist.' % (force_unicode(opts.verbose_name), escape(object_id)))
+            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
 
         # Populate deleted_objects, a data structure of all related objects that
         # will also be deleted.
@@ -689,12 +704,12 @@ class ModelAdmin(BaseModelAdmin):
         if request.POST: # The user has already confirmed the deletion.
             if perms_needed:
                 raise PermissionDenied
-            obj_display = str(obj)
-            obj.delete()
-            
+            obj_display = force_unicode(obj)
             self.log_deletion(request, obj, obj_display)
+            obj.delete()
+
             self.message_user(request, _('The %(name)s "%(obj)s" was deleted successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj_display)})
-            
+
             if not self.has_change_permission(request, None):
                 return HttpResponseRedirect("../../../../")
             return HttpResponseRedirect("../../")
@@ -738,8 +753,8 @@ class ModelAdmin(BaseModelAdmin):
         }
         context.update(extra_context or {})
         return render_to_response(self.object_history_template or [
-            "admin/%s/%s/object_history.html" % (opts.app_label, opts.object_name.lower()),
-            "admin/%s/object_history.html" % opts.app_label,
+            "admin/%s/%s/object_history.html" % (app_label, opts.object_name.lower()),
+            "admin/%s/object_history.html" % app_label,
             "admin/object_history.html"
         ], context, context_instance=template.RequestContext(request))
 
@@ -769,7 +784,7 @@ class InlineModelAdmin(BaseModelAdmin):
             self.verbose_name = self.model._meta.verbose_name
         if self.verbose_name_plural is None:
             self.verbose_name_plural = self.model._meta.verbose_name_plural
-    
+
     def _media(self):
         from django.conf import settings
         js = []
@@ -789,13 +804,15 @@ class InlineModelAdmin(BaseModelAdmin):
         if self.exclude is None:
             exclude = []
         else:
-            exclude = self.exclude
+            exclude = list(self.exclude)
+        # if exclude is an empty list we use None, since that's the actual
+        # default
         defaults = {
             "form": self.form,
             "formset": self.formset,
             "fk_name": self.fk_name,
             "fields": fields,
-            "exclude": exclude + kwargs.get("exclude", []),
+            "exclude": (exclude + kwargs.get("exclude", [])) or None,
             "formfield_callback": self.formfield_for_dbfield,
             "extra": self.extra,
             "max_num": self.max_num,
