@@ -1,15 +1,23 @@
 """
 Oracle database backend for Django.
 
-Requires cx_Oracle: http://www.python.net/crew/atuining/cx_Oracle/
+Requires cx_Oracle: http://cx-oracle.sourceforge.net/
 """
 
 import os
 import datetime
 import time
+try:
+    from decimal import Decimal
+except ImportError:
+    from django.utils._decimal import Decimal
 
 # Oracle takes client-side character set encoding from the environment.
 os.environ['NLS_LANG'] = '.UTF8'
+# This prevents unicode from getting mangled by getting encoded into the
+# potentially non-unicode database character set.
+os.environ['ORA_NCHAR_LITERAL_REPLACE'] = 'TRUE'
+
 try:
     import cx_Oracle as Database
 except ImportError, e:
@@ -32,9 +40,11 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     needs_datetime_string_cast = False
     uses_custom_query_class = True
     interprets_empty_strings_as_nulls = True
+    uses_savepoints = True
 
 
 class DatabaseOperations(BaseDatabaseOperations):
+
     def autoinc_sql(self, table, column):
         # To simulate auto-incrementing primary keys in Oracle, we have to
         # create a sequence and a trigger.
@@ -43,26 +53,26 @@ class DatabaseOperations(BaseDatabaseOperations):
         tbl_name = self.quote_name(table)
         col_name = self.quote_name(column)
         sequence_sql = """
-            DECLARE
-                i INTEGER;
-            BEGIN
-                SELECT COUNT(*) INTO i FROM USER_CATALOG
-                    WHERE TABLE_NAME = '%(sq_name)s' AND TABLE_TYPE = 'SEQUENCE';
-                IF i = 0 THEN
-                    EXECUTE IMMEDIATE 'CREATE SEQUENCE %(sq_name)s';
-                END IF;
-            END;
-            /""" % locals()
+DECLARE
+    i INTEGER;
+BEGIN
+    SELECT COUNT(*) INTO i FROM USER_CATALOG
+        WHERE TABLE_NAME = '%(sq_name)s' AND TABLE_TYPE = 'SEQUENCE';
+    IF i = 0 THEN
+        EXECUTE IMMEDIATE 'CREATE SEQUENCE "%(sq_name)s"';
+    END IF;
+END;
+/""" % locals()
         trigger_sql = """
-            CREATE OR REPLACE TRIGGER %(tr_name)s
-            BEFORE INSERT ON %(tbl_name)s
-            FOR EACH ROW
-            WHEN (new.%(col_name)s IS NULL)
-                BEGIN
-                    SELECT %(sq_name)s.nextval
-                    INTO :new.%(col_name)s FROM dual;
-                END;
-                /""" % locals()
+CREATE OR REPLACE TRIGGER "%(tr_name)s"
+BEFORE INSERT ON %(tbl_name)s
+FOR EACH ROW
+WHEN (new.%(col_name)s IS NULL)
+    BEGIN
+        SELECT "%(sq_name)s".nextval
+        INTO :new.%(col_name)s FROM dual;
+    END;
+/""" % locals()
         return sequence_sql, trigger_sql
 
     def date_extract_sql(self, lookup_type, field_name):
@@ -94,8 +104,8 @@ class DatabaseOperations(BaseDatabaseOperations):
             return "%s"
 
     def last_insert_id(self, cursor, table_name, pk_name):
-        sq_name = util.truncate_name(table_name, self.max_name_length() - 3)
-        cursor.execute('SELECT %s_sq.currval FROM dual' % sq_name)
+        sq_name = get_sequence_name(table_name)
+        cursor.execute('SELECT "%s".currval FROM dual' % sq_name)
         return cursor.fetchone()[0]
 
     def lookup_cast(self, lookup_type):
@@ -109,6 +119,11 @@ class DatabaseOperations(BaseDatabaseOperations):
     def prep_for_iexact_query(self, x):
         return x
 
+    def process_clob(self, value):
+        if value is None:
+            return u''
+        return force_unicode(value.read())
+
     def query_class(self, DefaultQueryClass):
         return query.query_class(DefaultQueryClass, Database)
 
@@ -118,7 +133,8 @@ class DatabaseOperations(BaseDatabaseOperations):
         # always defaults to uppercase.
         # We simplify things by making Oracle identifiers always uppercase.
         if not name.startswith('"') and not name.endswith('"'):
-            name = '"%s"' % util.truncate_name(name.upper(), self.max_name_length())
+            name = '"%s"' % util.truncate_name(name.upper(),
+                                               self.max_name_length())
         return name.upper()
 
     def random_function_sql(self):
@@ -141,6 +157,12 @@ class DatabaseOperations(BaseDatabaseOperations):
         connection.cursor()
         return connection.ops.regex_lookup(lookup_type)
 
+    def savepoint_create_sql(self, sid):
+        return "SAVEPOINT " + self.quote_name(sid)
+
+    def savepoint_rollback_sql(self, sid):
+        return "ROLLBACK TO SAVEPOINT " + self.quote_name(sid)
+
     def sql_flush(self, style, tables, sequences):
         # Return a list of 'TRUNCATE x;', 'TRUNCATE y;',
         # 'TRUNCATE z;'... style SQL statements
@@ -150,8 +172,8 @@ class DatabaseOperations(BaseDatabaseOperations):
             sql = ['%s %s %s;' % \
                     (style.SQL_KEYWORD('DELETE'),
                      style.SQL_KEYWORD('FROM'),
-                     style.SQL_FIELD(self.quote_name(table))
-                     ) for table in tables]
+                     style.SQL_FIELD(self.quote_name(table)))
+                    for table in tables]
             # Since we've just deleted all the rows, running our sequence
             # ALTER code will reset the sequence to 0.
             for sequence_info in sequences:
@@ -179,21 +201,25 @@ class DatabaseOperations(BaseDatabaseOperations):
                     output.append(query % {'sequence': sequence_name,
                                            'table': table_name,
                                            'column': column_name})
-                    break # Only one AutoField is allowed per model, so don't bother continuing.
+                    # Only one AutoField is allowed per model, so don't
+                    # continue to loop
+                    break
             for f in model._meta.many_to_many:
-                table_name = self.quote_name(f.m2m_db_table())
-                sequence_name = get_sequence_name(f.m2m_db_table())
-                column_name = self.quote_name('id')
-                output.append(query % {'sequence': sequence_name,
-                                       'table': table_name,
-                                       'column': column_name})
+                if not f.rel.through:
+                    table_name = self.quote_name(f.m2m_db_table())
+                    sequence_name = get_sequence_name(f.m2m_db_table())
+                    column_name = self.quote_name('id')
+                    output.append(query % {'sequence': sequence_name,
+                                           'table': table_name,
+                                           'column': column_name})
         return output
 
     def start_transaction_sql(self):
         return ''
 
     def tablespace_sql(self, tablespace, inline=False):
-        return "%sTABLESPACE %s" % ((inline and "USING INDEX " or ""), self.quote_name(tablespace))
+        return "%sTABLESPACE %s" % ((inline and "USING INDEX " or ""),
+            self.quote_name(tablespace))
 
     def value_to_db_time(self, value):
         if value is None:
@@ -240,21 +266,27 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     def _valid_connection(self):
         return self.connection is not None
 
+    def _connect_string(self, settings):
+        if len(settings.DATABASE_HOST.strip()) == 0:
+            settings.DATABASE_HOST = 'localhost'
+        if len(settings.DATABASE_PORT.strip()) != 0:
+            dsn = Database.makedsn(settings.DATABASE_HOST,
+                                   int(settings.DATABASE_PORT),
+                                   settings.DATABASE_NAME)
+        else:
+            dsn = settings.DATABASE_NAME
+        return "%s/%s@%s" % (settings.DATABASE_USER,
+                             settings.DATABASE_PASSWORD, dsn)
+
     def _cursor(self, settings):
         cursor = None
         if not self._valid_connection():
-            if len(settings.DATABASE_HOST.strip()) == 0:
-                settings.DATABASE_HOST = 'localhost'
-            if len(settings.DATABASE_PORT.strip()) != 0:
-                dsn = Database.makedsn(settings.DATABASE_HOST, int(settings.DATABASE_PORT), settings.DATABASE_NAME)
-                self.connection = Database.connect(settings.DATABASE_USER, settings.DATABASE_PASSWORD, dsn, **self.options)
-            else:
-                conn_string = "%s/%s@%s" % (settings.DATABASE_USER, settings.DATABASE_PASSWORD, settings.DATABASE_NAME)
-                self.connection = Database.connect(conn_string, **self.options)
+            conn_string = self._connect_string(settings)
+            self.connection = Database.connect(conn_string, **self.options)
             cursor = FormatStylePlaceholderCursor(self.connection)
             # Set oracle date to ansi date format.  This only needs to execute
             # once when we create a new connection.
-            cursor.execute("ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD' "
+            cursor.execute("ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS' "
                            "NLS_TIMESTAMP_FORMAT = 'YYYY-MM-DD HH24:MI:SS.FF'")
             try:
                 self.oracle_version = int(self.connection.version.split('.')[0])
@@ -276,33 +308,36 @@ class DatabaseWrapper(BaseDatabaseWrapper):
                 pass
         if not cursor:
             cursor = FormatStylePlaceholderCursor(self.connection)
-        # Default arraysize of 1 is highly sub-optimal.
-        cursor.arraysize = 100
         return cursor
 
+    # Oracle doesn't support savepoint commits.  Ignore them.
+    def _savepoint_commit(self, sid):
+        pass
+
 
 class OracleParam(object):
     """
     Wrapper object for formatting parameters for Oracle. If the string
     representation of the value is large enough (greater than 4000 characters)
-    the input size needs to be set as NCLOB. Alternatively, if the parameter has
-    an `input_size` attribute, then the value of the `input_size` attribute will
-    be used instead. Otherwise, no input size will be set for the parameter when
-    executing the query.
+    the input size needs to be set as CLOB. Alternatively, if the parameter
+    has an `input_size` attribute, then the value of the `input_size` attribute
+    will be used instead. Otherwise, no input size will be set for the
+    parameter when executing the query.
     """
+
     def __init__(self, param, charset, strings_only=False):
         self.smart_str = smart_str(param, charset, strings_only)
         if hasattr(param, 'input_size'):
             # If parameter has `input_size` attribute, use that.
             self.input_size = param.input_size
         elif isinstance(param, basestring) and len(param) > 4000:
-            # Mark any string parameter greater than 4000 characters as an NCLOB.
-            self.input_size = Database.NCLOB
+            # Mark any string param greater than 4000 characters as a CLOB.
+            self.input_size = Database.CLOB
         else:
             self.input_size = None
 
 
-class FormatStylePlaceholderCursor(Database.Cursor):
+class FormatStylePlaceholderCursor(object):
     """
     Django uses "format" (e.g. '%s') style placeholders, but Oracle uses ":var"
     style. This fixes it -- but note that if you want to use a literal "%s" in
@@ -313,35 +348,26 @@ class FormatStylePlaceholderCursor(Database.Cursor):
     """
     charset = 'utf-8'
 
+    def __init__(self, connection):
+        self.cursor = connection.cursor()
+        # Necessary to retrieve decimal values without rounding error.
+        self.cursor.numbersAsStrings = True
+        # Default arraysize of 1 is highly sub-optimal.
+        self.cursor.arraysize = 100
+
     def _format_params(self, params):
-        if isinstance(params, dict):
-            result = {}
-            for key, value in params.items():
-                result[smart_str(key, self.charset)] = OracleParam(param, self.charset)
-            return result
-        else:
-            return tuple([OracleParam(p, self.charset, True) for p in params])
+        return tuple([OracleParam(p, self.charset, True) for p in params])
 
     def _guess_input_sizes(self, params_list):
-        if isinstance(params_list[0], dict):
-            sizes = {}
-            iterators = [params.iteritems() for params in params_list]
-        else:
-            sizes = [None] * len(params_list[0])
-            iterators = [enumerate(params) for params in params_list]
-        for iterator in iterators:
-            for key, value in iterator:
-                if value.input_size: sizes[key] = value.input_size
-        if isinstance(sizes, dict):
-            self.setinputsizes(**sizes)
-        else:
-            self.setinputsizes(*sizes)
+        sizes = [None] * len(params_list[0])
+        for params in params_list:
+            for i, value in enumerate(params):
+                if value.input_size:
+                    sizes[i] = value.input_size
+        self.setinputsizes(*sizes)
 
     def _param_generator(self, params):
-        if isinstance(params, dict):
-            return dict([(k, p.smart_str) for k, p in params.iteritems()])
-        else:
-            return [p.smart_str for p in params]
+        return [p.smart_str for p in params]
 
     def execute(self, query, params=None):
         if params is None:
@@ -358,19 +384,19 @@ class FormatStylePlaceholderCursor(Database.Cursor):
         query = smart_str(query, self.charset) % tuple(args)
         self._guess_input_sizes([params])
         try:
-            return Database.Cursor.execute(self, query, self._param_generator(params))
+            return self.cursor.execute(query, self._param_generator(params))
         except DatabaseError, e:
             # cx_Oracle <= 4.4.0 wrongly raises a DatabaseError for ORA-01400.
-            if e.message.code == 1400 and type(e) != IntegrityError:
-                e = IntegrityError(e.message)
+            if e.args[0].code == 1400 and not isinstance(e, IntegrityError):
+                e = IntegrityError(e.args[0])
             raise e
 
     def executemany(self, query, params=None):
         try:
-          args = [(':arg%d' % i) for i in range(len(params[0]))]
+            args = [(':arg%d' % i) for i in range(len(params[0]))]
         except (IndexError, TypeError):
-          # No params given, nothing to do
-          return None
+            # No params given, nothing to do
+            return None
         # cx_Oracle wants no trailing ';' for SQL statements.  For PL/SQL, it
         # it does want a trailing ';' but not a trailing '/'.  However, these
         # characters must be included in the original query in case the query
@@ -381,26 +407,79 @@ class FormatStylePlaceholderCursor(Database.Cursor):
         formatted = [self._format_params(i) for i in params]
         self._guess_input_sizes(formatted)
         try:
-            return Database.Cursor.executemany(self, query, [self._param_generator(p) for p in formatted])
+            return self.cursor.executemany(query,
+                                [self._param_generator(p) for p in formatted])
         except DatabaseError, e:
             # cx_Oracle <= 4.4.0 wrongly raises a DatabaseError for ORA-01400.
-            if e.message.code == 1400 and type(e) != IntegrityError:
-                e = IntegrityError(e.message)
+            if e.args[0].code == 1400 and not isinstance(e, IntegrityError):
+                e = IntegrityError(e.args[0])
             raise e
 
     def fetchone(self):
-        row = Database.Cursor.fetchone(self)
+        row = self.cursor.fetchone()
         if row is None:
             return row
-        return tuple([to_unicode(e) for e in row])
+        return self._rowfactory(row)
 
     def fetchmany(self, size=None):
         if size is None:
             size = self.arraysize
-        return tuple([tuple([to_unicode(e) for e in r]) for r in Database.Cursor.fetchmany(self, size)])
+        return tuple([self._rowfactory(r)
+                      for r in self.cursor.fetchmany(size)])
 
     def fetchall(self):
-        return tuple([tuple([to_unicode(e) for e in r]) for r in Database.Cursor.fetchall(self)])
+        return tuple([self._rowfactory(r)
+                      for r in self.cursor.fetchall()])
+
+    def _rowfactory(self, row):
+        # Cast numeric values as the appropriate Python type based upon the
+        # cursor description, and convert strings to unicode.
+        casted = []
+        for value, desc in zip(row, self.cursor.description):
+            if value is not None and desc[1] is Database.NUMBER:
+                precision, scale = desc[4:6]
+                if scale == -127:
+                    if precision == 0:
+                        # NUMBER column: decimal-precision floating point
+                        # This will normally be an integer from a sequence,
+                        # but it could be a decimal value.
+                        if '.' in value:
+                            value = Decimal(value)
+                        else:
+                            value = int(value)
+                    else:
+                        # FLOAT column: binary-precision floating point.
+                        # This comes from FloatField columns.
+                        value = float(value)
+                elif precision > 0:
+                    # NUMBER(p,s) column: decimal-precision fixed point.
+                    # This comes from IntField and DecimalField columns.
+                    if scale == 0:
+                        value = int(value)
+                    else:
+                        value = Decimal(value)
+                elif '.' in value:
+                    # No type information. This normally comes from a
+                    # mathematical expression in the SELECT list. Guess int
+                    # or Decimal based on whether it has a decimal point.
+                    value = Decimal(value)
+                else:
+                    value = int(value)
+            elif desc[1] in (Database.STRING, Database.FIXED_CHAR,
+                             Database.LONG_STRING):
+                value = to_unicode(value)
+            casted.append(value)
+        return tuple(casted)
+
+    def __getattr__(self, attr):
+        if attr in self.__dict__:
+            return self.__dict__[attr]
+        else:
+            return getattr(self.cursor, attr)
+
+    def __iter__(self):
+        return iter(self.cursor)
+
 
 def to_unicode(s):
     """
@@ -411,30 +490,33 @@ def to_unicode(s):
         return force_unicode(s)
     return s
 
+
 def _get_sequence_reset_sql():
     # TODO: colorize this SQL code with style.SQL_KEYWORD(), etc.
     return """
-        DECLARE
-            startvalue integer;
-            cval integer;
-        BEGIN
-            LOCK TABLE %(table)s IN SHARE MODE;
-            SELECT NVL(MAX(%(column)s), 0) INTO startvalue FROM %(table)s;
-            SELECT %(sequence)s.nextval INTO cval FROM dual;
-            cval := startvalue - cval;
-            IF cval != 0 THEN
-                EXECUTE IMMEDIATE 'ALTER SEQUENCE %(sequence)s MINVALUE 0 INCREMENT BY '||cval;
-                SELECT %(sequence)s.nextval INTO cval FROM dual;
-                EXECUTE IMMEDIATE 'ALTER SEQUENCE %(sequence)s INCREMENT BY 1';
-            END IF;
-            COMMIT;
-        END;
-        /"""
+DECLARE
+    startvalue integer;
+    cval integer;
+BEGIN
+    LOCK TABLE %(table)s IN SHARE MODE;
+    SELECT NVL(MAX(%(column)s), 0) INTO startvalue FROM %(table)s;
+    SELECT "%(sequence)s".nextval INTO cval FROM dual;
+    cval := startvalue - cval;
+    IF cval != 0 THEN
+        EXECUTE IMMEDIATE 'ALTER SEQUENCE "%(sequence)s" MINVALUE 0 INCREMENT BY '||cval;
+        SELECT "%(sequence)s".nextval INTO cval FROM dual;
+        EXECUTE IMMEDIATE 'ALTER SEQUENCE "%(sequence)s" INCREMENT BY 1';
+    END IF;
+    COMMIT;
+END;
+/"""
+
 
 def get_sequence_name(table):
     name_length = DatabaseOperations().max_name_length() - 3
     return '%s_SQ' % util.truncate_name(table, name_length).upper()
 
+
 def get_trigger_name(table):
     name_length = DatabaseOperations().max_name_length() - 3
     return '%s_TR' % util.truncate_name(table, name_length).upper()
