@@ -19,7 +19,8 @@ try:
 except NameError:
     from django.utils.itercompat import sorted
 
-temp_storage = FileSystemStorage(tempfile.gettempdir())
+temp_storage_dir = tempfile.mkdtemp()
+temp_storage = FileSystemStorage(temp_storage_dir)
 
 ARTICLE_STATUS = (
     (1, 'Draft'),
@@ -92,12 +93,16 @@ class PhoneNumber(models.Model):
 
 class TextFile(models.Model):
     description = models.CharField(max_length=20)
-    file = models.FileField(storage=temp_storage, upload_to='tests')
+    file = models.FileField(storage=temp_storage, upload_to='tests', max_length=15)
 
     def __unicode__(self):
         return self.description
 
 class ImageFile(models.Model):
+    def custom_upload_path(self, filename):
+        path = self.path or 'tests'
+        return '%s/%s' % (path, filename)
+
     description = models.CharField(max_length=20)
     try:
         # If PIL is available, try testing PIL.
@@ -105,9 +110,10 @@ class ImageFile(models.Model):
         # for PyPy, you need to check for the underlying modules
         # If PIL is not available, this test is equivalent to TextFile above.
         from PIL import Image, _imaging
-        image = models.ImageField(storage=temp_storage, upload_to='tests')
+        image = models.ImageField(storage=temp_storage, upload_to=custom_upload_path)
     except ImportError:
-        image = models.FileField(storage=temp_storage, upload_to='tests')
+        image = models.FileField(storage=temp_storage, upload_to=custom_upload_path)
+    path = models.CharField(max_length=16, blank=True, default='')
 
     def __unicode__(self):
         return self.description
@@ -144,7 +150,33 @@ class Inventory(models.Model):
 
    def __unicode__(self):
       return self.name
-      
+
+class Book(models.Model):
+    title = models.CharField(max_length=40)
+    author = models.ForeignKey(Writer, blank=True, null=True)
+    special_id = models.IntegerField(blank=True, null=True, unique=True)
+
+    class Meta:
+        unique_together = ('title', 'author')
+
+class ExplicitPK(models.Model):
+    key = models.CharField(max_length=20, primary_key=True)
+    desc = models.CharField(max_length=20, blank=True, unique=True)
+    class Meta:
+        unique_together = ('key', 'desc')
+
+    def __unicode__(self):
+        return self.key
+
+class Post(models.Model):
+    title = models.CharField(max_length=50, unique_for_date='posted', blank=True)
+    slug = models.CharField(max_length=50, unique_for_year='posted', blank=True)
+    subtitle = models.CharField(max_length=50, unique_for_month='posted', blank=True)
+    posted = models.DateField()
+
+    def __unicode__(self):
+        return self.name
+
 __test__ = {'API_TESTS': """
 >>> from django import forms
 >>> from django.forms.models import ModelForm, model_to_dict
@@ -170,6 +202,17 @@ Extra fields.
 >>> CategoryForm.base_fields.keys()
 ['name', 'slug', 'url', 'some_extra_field']
 
+Extra field that has a name collision with a related object accessor.
+
+>>> class WriterForm(ModelForm):
+...     book = forms.CharField(required=False)
+...
+...     class Meta:
+...         model = Writer
+
+>>> wf = WriterForm({'name': 'Richard Lockridge'})
+>>> wf.is_valid()
+True
 
 Replacing a field.
 
@@ -526,6 +569,30 @@ Add some categories and test the many-to-many form output.
 <option value="3">Third test</option>
 </select>  Hold down "Control", or "Command" on a Mac, to select more than one.</li>
 
+Initial values can be provided for model forms
+>>> f = TestArticleForm(auto_id=False, initial={'headline': 'Your headline here', 'categories': ['1','2']})
+>>> print f.as_ul()
+<li>Headline: <input type="text" name="headline" value="Your headline here" maxlength="50" /></li>
+<li>Slug: <input type="text" name="slug" maxlength="50" /></li>
+<li>Pub date: <input type="text" name="pub_date" /></li>
+<li>Writer: <select name="writer">
+<option value="" selected="selected">---------</option>
+<option value="1">Mike Royko</option>
+<option value="2">Bob Woodward</option>
+</select></li>
+<li>Article: <textarea rows="10" cols="40" name="article"></textarea></li>
+<li>Status: <select name="status">
+<option value="" selected="selected">---------</option>
+<option value="1">Draft</option>
+<option value="2">Pending</option>
+<option value="3">Live</option>
+</select></li>
+<li>Categories: <select multiple="multiple" name="categories">
+<option value="1" selected="selected">Entertainment</option>
+<option value="2" selected="selected">It&#39;s a test</option>
+<option value="3">Third test</option>
+</select>  Hold down "Control", or "Command" on a Mac, to select more than one.</li>
+
 >>> f = TestArticleForm({'headline': u'New headline', 'slug': u'new-headline', 'pub_date': u'1988-01-04',
 ...     'writer': u'1', 'article': u'Hello.', 'categories': [u'1', u'2']}, instance=new_art)
 >>> new_art = f.save()
@@ -779,6 +846,10 @@ ValidationError: [u'Select a valid choice. 100 is not one of the available choic
 Traceback (most recent call last):
 ...
 ValidationError: [u'Enter a list of values.']
+>>> f.clean(['fail'])
+Traceback (most recent call last):
+...
+ValidationError: [u'"fail" is not a valid value for a primary key.']
 
 # Add a Category object *after* the ModelMultipleChoiceField has already been
 # instantiated. This proves clean() checks the database during clean() rather
@@ -940,6 +1011,11 @@ True
 >>> instance.file
 <FieldFile: tests/test1.txt>
 
+# Check if the max_length attribute has been inherited from the model.
+>>> f = TextFileForm(data={'description': u'Assistance'}, files={'file': SimpleUploadedFile('test-maxlength.txt', 'hello world')})
+>>> f.is_valid()
+False
+
 # Edit an instance that already has the file defined in the model. This will not
 # save the file again, but leave it exactly as it is.
 
@@ -1113,6 +1189,15 @@ True
 <...FieldFile: tests/test3.png>
 >>> instance.delete()
 
+# Test callable upload_to behavior that's dependent on the value of another field in the model
+>>> f = ImageFileForm(data={'description': u'And a final one', 'path': 'foo'}, files={'image': SimpleUploadedFile('test4.png', image_data)})
+>>> f.is_valid()
+True
+>>> instance = f.save()
+>>> instance.image
+<...FieldFile: foo/test4.png>
+>>> instance.delete()
+
 # Media on a ModelForm ########################################################
 
 # Similar to a regular Form class you can define custom media to be used on
@@ -1200,6 +1285,51 @@ False
 >>> form.is_valid()
 True
 
+# Unique & unique together with null values
+>>> class BookForm(ModelForm):
+...     class Meta:
+...        model = Book
+>>> w = Writer.objects.get(name='Mike Royko')
+>>> form = BookForm({'title': 'I May Be Wrong But I Doubt It', 'author' : w.pk})
+>>> form.is_valid()
+True
+>>> form.save()
+<Book: Book object>
+>>> form = BookForm({'title': 'I May Be Wrong But I Doubt It', 'author' : w.pk})
+>>> form.is_valid()
+False
+>>> form._errors
+{'__all__': [u'Book with this Title and Author already exists.']}
+>>> form = BookForm({'title': 'I May Be Wrong But I Doubt It'})
+>>> form.is_valid()
+True
+>>> form.save()
+<Book: Book object>
+>>> form = BookForm({'title': 'I May Be Wrong But I Doubt It'})
+>>> form.is_valid()
+True
+
+# Test for primary_key being in the form and failing validation.
+>>> class ExplicitPKForm(ModelForm):
+...     class Meta:
+...         model = ExplicitPK
+...         fields = ('key', 'desc',)
+>>> form = ExplicitPKForm({'key': u'', 'desc': u'' })
+>>> form.is_valid()
+False
+
+# Ensure keys and blank character strings are tested for uniqueness.
+>>> form = ExplicitPKForm({'key': u'key1', 'desc': u''})
+>>> form.is_valid()
+True
+>>> form.save()
+<ExplicitPK: key1>
+>>> form = ExplicitPKForm({'key': u'key1', 'desc': u''})
+>>> form.is_valid()
+False
+>>> sorted(form.errors.items())
+[('__all__', [u'Explicit pk with this Key and Desc already exists.']), ('desc', [u'Explicit pk with this Desc already exists.']), ('key', [u'Explicit pk with this Key already exists.'])]
+
 # Choices on CharField and IntegerField
 >>> class ArticleForm(ModelForm):
 ...     class Meta:
@@ -1251,4 +1381,40 @@ ValidationError: [u'Select a valid choice. z is not one of the available choices
 >>> core = form.save()
 >>> core.parent
 <Inventory: Pear>
+
+### Validation on unique_for_date
+
+>>> p = Post.objects.create(title="Django 1.0 is released", slug="Django 1.0", subtitle="Finally", posted=datetime.date(2008, 9, 3))
+>>> class PostForm(ModelForm):
+...     class Meta:
+...         model = Post
+
+>>> f = PostForm({'title': "Django 1.0 is released", 'posted': '2008-09-03'})
+>>> f.is_valid()
+False
+>>> f.errors
+{'title': [u'Title must be unique for Posted date.']}
+>>> f = PostForm({'title': "Work on Django 1.1 begins", 'posted': '2008-09-03'})
+>>> f.is_valid()
+True
+>>> f = PostForm({'title': "Django 1.0 is released", 'posted': '2008-09-04'})
+>>> f.is_valid()
+True
+>>> f = PostForm({'slug': "Django 1.0", 'posted': '2008-01-01'})
+>>> f.is_valid()
+False
+>>> f.errors
+{'slug': [u'Slug must be unique for Posted year.']}
+>>> f = PostForm({'subtitle': "Finally", 'posted': '2008-09-30'})
+>>> f.is_valid()
+False
+>>> f.errors
+{'subtitle': [u'Subtitle must be unique for Posted month.']}
+>>> f = PostForm({'subtitle': "Finally", "title": "Django 1.0 is released", "slug": "Django 1.0", 'posted': '2008-09-03'}, instance=p)
+>>> f.is_valid()
+True
+
+# Clean up
+>>> import shutil
+>>> shutil.rmtree(temp_storage_dir)
 """}
