@@ -25,9 +25,29 @@ class Book(models.Model):
     author = models.ForeignKey(Author)
     title = models.CharField(max_length=100)
 
+    class Meta:
+        unique_together = (
+            ('author', 'title'),
+        )
+        ordering = ['id']
+
     def __unicode__(self):
         return self.title
 
+class BookWithCustomPK(models.Model):
+    my_pk = models.DecimalField(max_digits=5, decimal_places=0, primary_key=True)
+    author = models.ForeignKey(Author)
+    title = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return u'%s: %s' % (self.my_pk, self.title)
+
+class AlternateBook(Book):
+    notes = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return u'%s - %s' % (self.title, self.notes)
+
 class AuthorMeeting(models.Model):
     name = models.CharField(max_length=100)
     authors = models.ManyToManyField(Author)
@@ -46,7 +66,7 @@ class CustomPrimaryKey(models.Model):
 class Place(models.Model):
     name = models.CharField(max_length=50)
     city = models.CharField(max_length=50)
-    
+
     def __unicode__(self):
         return self.name
 
@@ -54,7 +74,7 @@ class Owner(models.Model):
     auto_id = models.AutoField(primary_key=True)
     name = models.CharField(max_length=100)
     place = models.ForeignKey(Place)
-    
+
     def __unicode__(self):
         return "%s at %s" % (self.name, self.place)
 
@@ -67,13 +87,13 @@ class Location(models.Model):
 class OwnerProfile(models.Model):
     owner = models.OneToOneField(Owner, primary_key=True)
     age = models.PositiveIntegerField()
-    
+
     def __unicode__(self):
         return "%s is %d" % (self.owner.name, self.age)
 
 class Restaurant(Place):
     serves_pizza = models.BooleanField()
-    
+
     def __unicode__(self):
         return self.name
 
@@ -96,6 +116,24 @@ class Price(models.Model):
 class MexicanRestaurant(Restaurant):
     serves_tacos = models.BooleanField()
 
+# models for testing unique_together validation when a fk is involved and
+# using inlineformset_factory.
+class Repository(models.Model):
+    name = models.CharField(max_length=25)
+
+    def __unicode__(self):
+        return self.name
+
+class Revision(models.Model):
+    repository = models.ForeignKey(Repository)
+    revision = models.CharField(max_length=40)
+
+    class Meta:
+        unique_together = (("repository", "revision"),)
+
+    def __unicode__(self):
+        return u"%s (%s)" % (self.revision, unicode(self.repository))
+
 # models for testing callable defaults (see bug #7975). If you define a model
 # with a callable default value, you cannot rely on the initial value in a
 # form.
@@ -107,6 +145,39 @@ class Membership(models.Model):
     date_joined = models.DateTimeField(default=datetime.datetime.now)
     karma = models.IntegerField()
 
+# models for testing a null=True fk to a parent
+class Team(models.Model):
+    name = models.CharField(max_length=100)
+
+class Player(models.Model):
+    team = models.ForeignKey(Team, null=True)
+    name = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return self.name
+
+class Poet(models.Model):
+    name = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return self.name
+
+class Poem(models.Model):
+    poet = models.ForeignKey(Poet)
+    name = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return self.name
+
+class Post(models.Model):
+    title = models.CharField(max_length=50, unique_for_date='posted', blank=True)
+    slug = models.CharField(max_length=50, unique_for_year='posted', blank=True)
+    subtitle = models.CharField(max_length=50, unique_for_month='posted', blank=True)
+    posted = models.DateField()
+
+    def __unicode__(self):
+        return self.name
+
 __test__ = {'API_TESTS': """
 
 >>> from datetime import date
@@ -364,9 +435,9 @@ admin system's edit inline functionality works.
 >>> formset = AuthorBooksFormSet(instance=author)
 >>> for form in formset.forms:
 ...     print form.as_p()
-<p><label for="id_book_set-0-title">Title:</label> <input id="id_book_set-0-title" type="text" name="book_set-0-title" maxlength="100" /><input type="hidden" name="book_set-0-id" id="id_book_set-0-id" /></p>
-<p><label for="id_book_set-1-title">Title:</label> <input id="id_book_set-1-title" type="text" name="book_set-1-title" maxlength="100" /><input type="hidden" name="book_set-1-id" id="id_book_set-1-id" /></p>
-<p><label for="id_book_set-2-title">Title:</label> <input id="id_book_set-2-title" type="text" name="book_set-2-title" maxlength="100" /><input type="hidden" name="book_set-2-id" id="id_book_set-2-id" /></p>
+<p><label for="id_book_set-0-title">Title:</label> <input id="id_book_set-0-title" type="text" name="book_set-0-title" maxlength="100" /><input type="hidden" name="book_set-0-author" value="1" id="id_book_set-0-author" /><input type="hidden" name="book_set-0-id" id="id_book_set-0-id" /></p>
+<p><label for="id_book_set-1-title">Title:</label> <input id="id_book_set-1-title" type="text" name="book_set-1-title" maxlength="100" /><input type="hidden" name="book_set-1-author" value="1" id="id_book_set-1-author" /><input type="hidden" name="book_set-1-id" id="id_book_set-1-id" /></p>
+<p><label for="id_book_set-2-title">Title:</label> <input id="id_book_set-2-title" type="text" name="book_set-2-title" maxlength="100" /><input type="hidden" name="book_set-2-author" value="1" id="id_book_set-2-author" /><input type="hidden" name="book_set-2-id" id="id_book_set-2-id" /></p>
 
 >>> data = {
 ...     'book_set-TOTAL_FORMS': '3', # the number of forms rendered
@@ -398,9 +469,9 @@ book.
 >>> formset = AuthorBooksFormSet(instance=author)
 >>> for form in formset.forms:
 ...     print form.as_p()
-<p><label for="id_book_set-0-title">Title:</label> <input id="id_book_set-0-title" type="text" name="book_set-0-title" value="Les Fleurs du Mal" maxlength="100" /><input type="hidden" name="book_set-0-id" value="1" id="id_book_set-0-id" /></p>
-<p><label for="id_book_set-1-title">Title:</label> <input id="id_book_set-1-title" type="text" name="book_set-1-title" maxlength="100" /><input type="hidden" name="book_set-1-id" id="id_book_set-1-id" /></p>
-<p><label for="id_book_set-2-title">Title:</label> <input id="id_book_set-2-title" type="text" name="book_set-2-title" maxlength="100" /><input type="hidden" name="book_set-2-id" id="id_book_set-2-id" /></p>
+<p><label for="id_book_set-0-title">Title:</label> <input id="id_book_set-0-title" type="text" name="book_set-0-title" value="Les Fleurs du Mal" maxlength="100" /><input type="hidden" name="book_set-0-author" value="1" id="id_book_set-0-author" /><input type="hidden" name="book_set-0-id" value="1" id="id_book_set-0-id" /></p>
+<p><label for="id_book_set-1-title">Title:</label> <input id="id_book_set-1-title" type="text" name="book_set-1-title" maxlength="100" /><input type="hidden" name="book_set-1-author" value="1" id="id_book_set-1-author" /><input type="hidden" name="book_set-1-id" id="id_book_set-1-id" /></p>
+<p><label for="id_book_set-2-title">Title:</label> <input id="id_book_set-2-title" type="text" name="book_set-2-title" maxlength="100" /><input type="hidden" name="book_set-2-author" value="1" id="id_book_set-2-author" /><input type="hidden" name="book_set-2-id" id="id_book_set-2-id" /></p>
 
 >>> data = {
 ...     'book_set-TOTAL_FORMS': '3', # the number of forms rendered
@@ -443,7 +514,7 @@ This is used in the admin for save_as functionality.
 True
 
 >>> new_author = Author.objects.create(name='Charles Baudelaire')
->>> formset.instance = new_author
+>>> formset = AuthorBooksFormSet(data, instance=new_author, save_as_new=True)
 >>> [book for book in formset.save() if book.author.pk == new_author.pk]
 [<Book: Les Fleurs du Mal>, <Book: Le Spleen de Paris>]
 
@@ -452,8 +523,63 @@ Test using a custom prefix on an inline formset.
 >>> formset = AuthorBooksFormSet(prefix="test")
 >>> for form in formset.forms:
 ...     print form.as_p()
-<p><label for="id_test-0-title">Title:</label> <input id="id_test-0-title" type="text" name="test-0-title" maxlength="100" /><input type="hidden" name="test-0-id" id="id_test-0-id" /></p>
-<p><label for="id_test-1-title">Title:</label> <input id="id_test-1-title" type="text" name="test-1-title" maxlength="100" /><input type="hidden" name="test-1-id" id="id_test-1-id" /></p>
+<p><label for="id_test-0-title">Title:</label> <input id="id_test-0-title" type="text" name="test-0-title" maxlength="100" /><input type="hidden" name="test-0-author" id="id_test-0-author" /><input type="hidden" name="test-0-id" id="id_test-0-id" /></p>
+<p><label for="id_test-1-title">Title:</label> <input id="id_test-1-title" type="text" name="test-1-title" maxlength="100" /><input type="hidden" name="test-1-author" id="id_test-1-author" /><input type="hidden" name="test-1-id" id="id_test-1-id" /></p>
+
+Test inline formsets where the inline-edited object has a custom primary key that is not the fk to the parent object.
+
+>>> AuthorBooksFormSet2 = inlineformset_factory(Author, BookWithCustomPK, can_delete=False, extra=1)
+
+>>> formset = AuthorBooksFormSet2(instance=author)
+>>> for form in formset.forms:
+...     print form.as_p()
+<p><label for="id_bookwithcustompk_set-0-my_pk">My pk:</label> <input type="text" name="bookwithcustompk_set-0-my_pk" id="id_bookwithcustompk_set-0-my_pk" /></p>
+    <p><label for="id_bookwithcustompk_set-0-title">Title:</label> <input id="id_bookwithcustompk_set-0-title" type="text" name="bookwithcustompk_set-0-title" maxlength="100" /><input type="hidden" name="bookwithcustompk_set-0-author" value="1" id="id_bookwithcustompk_set-0-author" /></p>
+
+>>> data = {
+...     'bookwithcustompk_set-TOTAL_FORMS': '1', # the number of forms rendered
+...     'bookwithcustompk_set-INITIAL_FORMS': '0', # the number of forms with initial data
+...     'bookwithcustompk_set-0-my_pk': '77777',
+...     'bookwithcustompk_set-0-title': 'Les Fleurs du Mal',
+... }
+
+>>> formset = AuthorBooksFormSet2(data, instance=author)
+>>> formset.is_valid()
+True
+
+>>> formset.save()
+[<BookWithCustomPK: 77777: Les Fleurs du Mal>]
+
+>>> for book in author.bookwithcustompk_set.all():
+...     print book.title
+Les Fleurs du Mal
+
+Test inline formsets where the inline-edited object uses multi-table inheritance, thus
+has a non AutoField yet auto-created primary key.
+
+>>> AuthorBooksFormSet3 = inlineformset_factory(Author, AlternateBook, can_delete=False, extra=1)
+
+>>> formset = AuthorBooksFormSet3(instance=author)
+>>> for form in formset.forms:
+...     print form.as_p()
+<p><label for="id_alternatebook_set-0-title">Title:</label> <input id="id_alternatebook_set-0-title" type="text" name="alternatebook_set-0-title" maxlength="100" /></p>
+    <p><label for="id_alternatebook_set-0-notes">Notes:</label> <input id="id_alternatebook_set-0-notes" type="text" name="alternatebook_set-0-notes" maxlength="100" /><input type="hidden" name="alternatebook_set-0-author" value="1" id="id_alternatebook_set-0-author" /><input type="hidden" name="alternatebook_set-0-book_ptr" id="id_alternatebook_set-0-book_ptr" /></p>
+
+
+>>> data = {
+...     'alternatebook_set-TOTAL_FORMS': '1', # the number of forms rendered
+...     'alternatebook_set-INITIAL_FORMS': '0', # the number of forms with initial data
+...     'alternatebook_set-0-title': 'Flowers of Evil',
+...     'alternatebook_set-0-notes': 'English translation of Les Fleurs du Mal'
+... }
+
+>>> formset = AuthorBooksFormSet3(data, instance=author)
+>>> formset.is_valid()
+True
+
+>>> formset.save()
+[<AlternateBook: Flowers of Evil - English translation of Les Fleurs du Mal>]
+
 
 # Test a custom primary key ###################################################
 
@@ -475,8 +601,8 @@ We need to ensure that it is displayed
 >>> formset = FormSet(instance=place)
 >>> for form in formset.forms:
 ...     print form.as_p()
-<p><label for="id_owner_set-0-name">Name:</label> <input id="id_owner_set-0-name" type="text" name="owner_set-0-name" maxlength="100" /><input type="hidden" name="owner_set-0-auto_id" id="id_owner_set-0-auto_id" /></p>
-<p><label for="id_owner_set-1-name">Name:</label> <input id="id_owner_set-1-name" type="text" name="owner_set-1-name" maxlength="100" /><input type="hidden" name="owner_set-1-auto_id" id="id_owner_set-1-auto_id" /></p>
+<p><label for="id_owner_set-0-name">Name:</label> <input id="id_owner_set-0-name" type="text" name="owner_set-0-name" maxlength="100" /><input type="hidden" name="owner_set-0-place" value="1" id="id_owner_set-0-place" /><input type="hidden" name="owner_set-0-auto_id" id="id_owner_set-0-auto_id" /></p>
+<p><label for="id_owner_set-1-name">Name:</label> <input id="id_owner_set-1-name" type="text" name="owner_set-1-name" maxlength="100" /><input type="hidden" name="owner_set-1-place" value="1" id="id_owner_set-1-place" /><input type="hidden" name="owner_set-1-auto_id" id="id_owner_set-1-auto_id" /></p>
 
 >>> data = {
 ...     'owner_set-TOTAL_FORMS': '2',
@@ -495,9 +621,9 @@ True
 >>> formset = FormSet(instance=place)
 >>> for form in formset.forms:
 ...     print form.as_p()
-<p><label for="id_owner_set-0-name">Name:</label> <input id="id_owner_set-0-name" type="text" name="owner_set-0-name" value="Joe Perry" maxlength="100" /><input type="hidden" name="owner_set-0-auto_id" value="1" id="id_owner_set-0-auto_id" /></p>
-<p><label for="id_owner_set-1-name">Name:</label> <input id="id_owner_set-1-name" type="text" name="owner_set-1-name" maxlength="100" /><input type="hidden" name="owner_set-1-auto_id" id="id_owner_set-1-auto_id" /></p>
-<p><label for="id_owner_set-2-name">Name:</label> <input id="id_owner_set-2-name" type="text" name="owner_set-2-name" maxlength="100" /><input type="hidden" name="owner_set-2-auto_id" id="id_owner_set-2-auto_id" /></p>
+<p><label for="id_owner_set-0-name">Name:</label> <input id="id_owner_set-0-name" type="text" name="owner_set-0-name" value="Joe Perry" maxlength="100" /><input type="hidden" name="owner_set-0-place" value="1" id="id_owner_set-0-place" /><input type="hidden" name="owner_set-0-auto_id" value="1" id="id_owner_set-0-auto_id" /></p>
+<p><label for="id_owner_set-1-name">Name:</label> <input id="id_owner_set-1-name" type="text" name="owner_set-1-name" maxlength="100" /><input type="hidden" name="owner_set-1-place" value="1" id="id_owner_set-1-place" /><input type="hidden" name="owner_set-1-auto_id" id="id_owner_set-1-auto_id" /></p>
+<p><label for="id_owner_set-2-name">Name:</label> <input id="id_owner_set-2-name" type="text" name="owner_set-2-name" maxlength="100" /><input type="hidden" name="owner_set-2-place" value="1" id="id_owner_set-2-place" /><input type="hidden" name="owner_set-2-auto_id" id="id_owner_set-2-auto_id" /></p>
 
 >>> data = {
 ...     'owner_set-TOTAL_FORMS': '3',
@@ -534,7 +660,7 @@ True
 >>> formset = FormSet(instance=owner)
 >>> for form in formset.forms:
 ...     print form.as_p()
-<p><label for="id_ownerprofile-0-age">Age:</label> <input type="text" name="ownerprofile-0-age" id="id_ownerprofile-0-age" /><input type="hidden" name="ownerprofile-0-owner" id="id_ownerprofile-0-owner" /></p>
+<p><label for="id_ownerprofile-0-age">Age:</label> <input type="text" name="ownerprofile-0-age" id="id_ownerprofile-0-age" /><input type="hidden" name="ownerprofile-0-owner" value="1" id="id_ownerprofile-0-owner" /></p>
 
 >>> data = {
 ...     'ownerprofile-TOTAL_FORMS': '1',
@@ -565,14 +691,14 @@ True
 >>> formset.save()
 [<OwnerProfile: Joe Perry is 55>]
 
-# ForeignKey with unique=True should enforce max_num=1 
+# ForeignKey with unique=True should enforce max_num=1
 
 >>> FormSet = inlineformset_factory(Place, Location, can_delete=False)
 >>> formset = FormSet(instance=place)
 >>> for form in formset.forms:
 ...     print form.as_p()
 <p><label for="id_location_set-0-lat">Lat:</label> <input id="id_location_set-0-lat" type="text" name="location_set-0-lat" maxlength="100" /></p>
-<p><label for="id_location_set-0-lon">Lon:</label> <input id="id_location_set-0-lon" type="text" name="location_set-0-lon" maxlength="100" /><input type="hidden" name="location_set-0-id" id="id_location_set-0-id" /></p>
+<p><label for="id_location_set-0-lon">Lon:</label> <input id="id_location_set-0-lon" type="text" name="location_set-0-lon" maxlength="100" /><input type="hidden" name="location_set-0-place" value="1" id="id_location_set-0-place" /><input type="hidden" name="location_set-0-id" id="id_location_set-0-id" /></p>
 
 # Foreign keys in parents ########################################
 
@@ -635,6 +761,53 @@ False
 >>> formset.errors
 [{'__all__': [u'Price with this Price and Quantity already exists.']}]
 
+# unique_together with inlineformset_factory
+# Also see bug #8882.
+
+>>> repository = Repository.objects.create(name=u'Test Repo')
+>>> FormSet = inlineformset_factory(Repository, Revision, extra=1)
+>>> data = {
+...     'revision_set-TOTAL_FORMS': '1',
+...     'revision_set-INITIAL_FORMS': '0',
+...     'revision_set-0-repository': repository.pk,
+...     'revision_set-0-revision': '146239817507f148d448db38840db7c3cbf47c76',
+...     'revision_set-0-DELETE': '',
+... }
+>>> formset = FormSet(data, instance=repository)
+>>> formset.is_valid()
+True
+>>> formset.save()
+[<Revision: 146239817507f148d448db38840db7c3cbf47c76 (Test Repo)>]
+
+# attempt to save the same revision against against the same repo.
+>>> data = {
+...     'revision_set-TOTAL_FORMS': '1',
+...     'revision_set-INITIAL_FORMS': '0',
+...     'revision_set-0-repository': repository.pk,
+...     'revision_set-0-revision': '146239817507f148d448db38840db7c3cbf47c76',
+...     'revision_set-0-DELETE': '',
+... }
+>>> formset = FormSet(data, instance=repository)
+>>> formset.is_valid()
+False
+>>> formset.errors
+[{'__all__': [u'Revision with this Repository and Revision already exists.']}]
+
+# unique_together with inlineformset_factory with overridden form fields
+# Also see #9494
+
+>>> FormSet = inlineformset_factory(Repository, Revision, fields=('revision',), extra=1)
+>>> data = {
+...     'revision_set-TOTAL_FORMS': '1',
+...     'revision_set-INITIAL_FORMS': '0',
+...     'revision_set-0-repository': repository.pk,
+...     'revision_set-0-revision': '146239817507f148d448db38840db7c3cbf47c76',
+...     'revision_set-0-DELETE': '',
+... }
+>>> formset = FormSet(data, instance=repository)
+>>> formset.is_valid()
+False
+
 # Use of callable defaults (see bug #7975).
 
 >>> person = Person.objects.create(name='Ringo')
@@ -649,7 +822,7 @@ False
 >>> now = form.fields['date_joined'].initial
 >>> print form.as_p()
 <p><label for="id_membership_set-0-date_joined">Date joined:</label> <input type="text" name="membership_set-0-date_joined" value="..." id="id_membership_set-0-date_joined" /><input type="hidden" name="initial-membership_set-0-date_joined" value="..." id="id_membership_set-0-date_joined" /></p>
-<p><label for="id_membership_set-0-karma">Karma:</label> <input type="text" name="membership_set-0-karma" id="id_membership_set-0-karma" /><input type="hidden" name="membership_set-0-id" id="id_membership_set-0-id" /></p>
+<p><label for="id_membership_set-0-karma">Karma:</label> <input type="text" name="membership_set-0-karma" id="id_membership_set-0-karma" /><input type="hidden" name="membership_set-0-person" value="1" id="id_membership_set-0-person" /><input type="hidden" name="membership_set-0-id" id="id_membership_set-0-id" /></p>
 
 # test for validation with callable defaults. Validations rely on hidden fields
 
@@ -701,4 +874,143 @@ False
 >>> formset.is_valid()
 True
 
+# inlineformset_factory tests with fk having null=True. see #9462.
+# create some data that will exbit the issue
+>>> team = Team.objects.create(name=u"Red Vipers")
+>>> Player(name="Timmy").save()
+>>> Player(name="Bobby", team=team).save()
+
+>>> PlayerInlineFormSet = inlineformset_factory(Team, Player)
+>>> formset = PlayerInlineFormSet()
+>>> formset.get_queryset()
+[]
+
+>>> formset = PlayerInlineFormSet(instance=team)
+>>> formset.get_queryset()
+[<Player: Bobby>]
+
+# Prevent duplicates from within the same formset
+>>> FormSet = modelformset_factory(Product, extra=2)
+>>> data = {
+...     'form-TOTAL_FORMS': 2,
+...     'form-INITIAL_FORMS': 0,
+...     'form-0-slug': 'red_car',
+...     'form-1-slug': 'red_car',
+... }
+>>> formset = FormSet(data)
+>>> formset.is_valid()
+False
+>>> formset._non_form_errors
+[u'Please correct the duplicate data for slug.']
+
+>>> FormSet = modelformset_factory(Price, extra=2)
+>>> data = {
+...     'form-TOTAL_FORMS': 2,
+...     'form-INITIAL_FORMS': 0,
+...     'form-0-price': '25',
+...     'form-0-quantity': '7',
+...     'form-1-price': '25',
+...     'form-1-quantity': '7',
+... }
+>>> formset = FormSet(data)
+>>> formset.is_valid()
+False
+>>> formset._non_form_errors
+[u'Please correct the duplicate data for price and quantity, which must be unique.']
+
+# only the price field is specified, this should skip any unique checks since the unique_together is not fulfilled.
+# this will fail with a KeyError if broken.
+>>> FormSet = modelformset_factory(Price, fields=("price",), extra=2)
+>>> data = {
+...     'form-TOTAL_FORMS': '2',
+...     'form-INITIAL_FORMS': '0',
+...     'form-0-price': '24',
+...     'form-1-price': '24',
+... }
+>>> formset = FormSet(data)
+>>> formset.is_valid()
+True
+
+>>> FormSet = inlineformset_factory(Author, Book, extra=0)
+>>> author = Author.objects.order_by('id')[0]
+>>> book_ids = author.book_set.values_list('id', flat=True)
+>>> data = {
+...     'book_set-TOTAL_FORMS': '2',
+...     'book_set-INITIAL_FORMS': '2',
+...
+...     'book_set-0-title': 'The 2008 Election',
+...     'book_set-0-author': str(author.id),
+...     'book_set-0-id': str(book_ids[0]),
+...
+...     'book_set-1-title': 'The 2008 Election',
+...     'book_set-1-author': str(author.id),
+...     'book_set-1-id': str(book_ids[1]),
+... }
+>>> formset = FormSet(data=data, instance=author)
+>>> formset.is_valid()
+False
+>>> formset._non_form_errors
+[u'Please correct the duplicate data for title.']
+>>> formset.errors
+[{}, {'__all__': u'Please correct the duplicate values below.'}]
+
+>>> FormSet = modelformset_factory(Post, extra=2)
+>>> data = {
+...     'form-TOTAL_FORMS': '2',
+...     'form-INITIAL_FORMS': '0',
+...
+...     'form-0-title': 'blah',
+...     'form-0-slug': 'Morning',
+...     'form-0-subtitle': 'foo',
+...     'form-0-posted': '2009-01-01',
+...     'form-1-title': 'blah',
+...     'form-1-slug': 'Morning in Prague',
+...     'form-1-subtitle': 'rawr',
+...     'form-1-posted': '2009-01-01'
+... }
+>>> formset = FormSet(data)
+>>> formset.is_valid()
+False
+>>> formset._non_form_errors
+[u'Please correct the duplicate data for title which must be unique for the date in posted.']
+>>> formset.errors
+[{}, {'__all__': u'Please correct the duplicate values below.'}]
+
+>>> data = {
+...     'form-TOTAL_FORMS': '2',
+...     'form-INITIAL_FORMS': '0',
+...
+...     'form-0-title': 'foo',
+...     'form-0-slug': 'Morning in Prague',
+...     'form-0-subtitle': 'foo',
+...     'form-0-posted': '2009-01-01',
+...     'form-1-title': 'blah',
+...     'form-1-slug': 'Morning in Prague',
+...     'form-1-subtitle': 'rawr',
+...     'form-1-posted': '2009-08-02'
+... }
+>>> formset = FormSet(data)
+>>> formset.is_valid()
+False
+>>> formset._non_form_errors
+[u'Please correct the duplicate data for slug which must be unique for the year in posted.']
+
+>>> data = {
+...     'form-TOTAL_FORMS': '2',
+...     'form-INITIAL_FORMS': '0',
+...
+...     'form-0-title': 'foo',
+...     'form-0-slug': 'Morning in Prague',
+...     'form-0-subtitle': 'rawr',
+...     'form-0-posted': '2008-08-01',
+...     'form-1-title': 'blah',
+...     'form-1-slug': 'Prague',
+...     'form-1-subtitle': 'rawr',
+...     'form-1-posted': '2009-08-02'
+... }
+>>> formset = FormSet(data)
+>>> formset.is_valid()
+False
+>>> formset._non_form_errors
+[u'Please correct the duplicate data for subtitle which must be unique for the month in posted.']
 """}
