@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 Regression tests for the Test Client, especially the customized assertions.
 """
@@ -10,6 +11,13 @@ from django.core.exceptions import SuspiciousOperation
 from django.template import TemplateDoesNotExist, TemplateSyntaxError
 
 class AssertContainsTests(TestCase):
+    def setUp(self):
+        self.old_templates = settings.TEMPLATE_DIRS
+        settings.TEMPLATE_DIRS = (os.path.join(os.path.dirname(__file__), 'templates'),)
+
+    def tearDown(self):
+        settings.TEMPLATE_DIRS = self.old_templates
+
     def test_contains(self):
         "Responses can be inspected for content, including counting repeated substrings"
         response = self.client.get('/test_client_regress/no_template_view/')
@@ -56,6 +64,20 @@ class AssertContainsTests(TestCase):
         except AssertionError, e:
             self.assertEquals(str(e), "Found 0 instances of 'thrice' in response (expected 3)")
 
+    def test_unicode_contains(self):
+        "Unicode characters can be found in template context"
+        #Regression test for #10183
+        r = self.client.get('/test_client_regress/check_unicode/')
+        self.assertContains(r, u'さかき')
+        self.assertContains(r, '\xe5\xb3\xa0'.decode('utf-8'))
+
+    def test_unicode_not_contains(self):
+        "Unicode characters can be searched for, and not found in template context"
+        #Regression test for #10183
+        r = self.client.get('/test_client_regress/check_unicode/')
+        self.assertNotContains(r, u'はたけ')
+        self.assertNotContains(r, '\xe3\x81\xaf\xe3\x81\x9f\xe3\x81\x91'.decode('utf-8'))
+
 class AssertTemplateUsedTests(TestCase):
     fixtures = ['testdata.json']
 
@@ -309,17 +331,17 @@ class ExceptionTests(TestCase):
             self.client.get("/test_client_regress/staff_only/")
         except SuspiciousOperation:
             self.fail("Staff should be able to visit this page")
-    
+
 class TemplateExceptionTests(TestCase):
     def setUp(self):
         self.old_templates = settings.TEMPLATE_DIRS
         settings.TEMPLATE_DIRS = ()
-        
+
     def tearDown(self):
         settings.TEMPLATE_DIRS = self.old_templates
-        
+
     def test_no_404_template(self):
-        "Missing templates are correctly reported by test client"    
+        "Missing templates are correctly reported by test client"
         try:
             response = self.client.get("/no_such_view/")
             self.fail("Should get error about missing template")
@@ -334,7 +356,7 @@ class TemplateExceptionTests(TestCase):
             self.fail("Should get error about syntax error in template")
         except TemplateSyntaxError:
             pass
-        
+
 # We need two different tests to check URLconf substitution -  one to check
 # it was changed, and another one (without self.urls) to check it was reverted on
 # teardown. This pair of tests relies upon the alphabetical ordering of test execution.
@@ -342,7 +364,7 @@ class UrlconfSubstitutionTests(TestCase):
     urls = 'regressiontests.test_client_regress.urls'
 
     def test_urlconf_was_changed(self):
-        "TestCase can enforce a custom URLConf on a per-test basis"
+        "TestCase can enforce a custom URLconf on a per-test basis"
         url = reverse('arg_view', args=['somename'])
         self.assertEquals(url, '/arg_view/somename/')
 
@@ -382,4 +404,44 @@ class SessionTests(TestCase):
         response = self.client.get('/test_client_regress/check_session/')
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.content, 'YES')
-        
\ No newline at end of file
+
+    def test_logout(self):
+        """Logout should work whether the user is logged in or not (#9978)."""
+        self.client.logout()
+        login = self.client.login(username='testclient',password='password')
+        self.failUnless(login, 'Could not log in')
+        self.client.logout()
+        self.client.logout()
+
+class UnicodePayloadTests(TestCase):
+    def test_simple_unicode_payload(self):
+        "A simple ASCII-only unicode JSON document can be POSTed"
+        # Regression test for #10571
+        json = u'{"english": "mountain pass"}'
+        response = self.client.post("/test_client_regress/parse_unicode_json/", json,
+                                    content_type="application/json")
+        self.assertEqual(response.content, json)
+
+    def test_unicode_payload_utf8(self):
+        "A non-ASCII unicode data encoded as UTF-8 can be POSTed"
+        # Regression test for #10571
+        json = u'{"dog": "собака"}'
+        response = self.client.post("/test_client_regress/parse_unicode_json/", json,
+                                    content_type="application/json; charset=utf-8")
+        self.assertEqual(response.content, json.encode('utf-8'))
+
+    def test_unicode_payload_utf16(self):
+        "A non-ASCII unicode data encoded as UTF-16 can be POSTed"
+        # Regression test for #10571
+        json = u'{"dog": "собака"}'
+        response = self.client.post("/test_client_regress/parse_unicode_json/", json,
+                                    content_type="application/json; charset=utf-16")
+        self.assertEqual(response.content, json.encode('utf-16'))
+
+    def test_unicode_payload_non_utf(self):
+        "A non-ASCII unicode data as a non-UTF based encoding can be POSTed"
+        #Regression test for #10571
+        json = u'{"dog": "собака"}'
+        response = self.client.post("/test_client_regress/parse_unicode_json/", json,
+                                    content_type="application/json; charset=koi8-r")
+        self.assertEqual(response.content, json.encode('koi8-r'))
